'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Vue = _interopDefault(require('vue'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var DEFAULT_OBSERVER_OPTIONS = {
  rootMargin: '0px',
  threshold: 0
};

var Lazy = function () {
  function Lazy(Vue) {
    classCallCheck(this, Lazy);

    this.vue = Vue;
    this._observer = null;
    this.ListenerQueue = [];
    this.observerOptions = DEFAULT_OBSERVER_OPTIONS;
    this.init();
  }

  createClass(Lazy, [{
    key: 'init',
    value: function init() {
      this._observer = new IntersectionObserver(this._observerHandler.bind(this), this.observerOptions);
    }
  }, {
    key: 'add',
    value: function add(el, binding, vnode) {
      var _this = this;

      this.ListenerQueue.push({
        loaded: false,
        el: el,
        binding: binding,
        vnode: vnode
      });
      this.vue.nextTick(function () {
        _this._observer && _this._observer.observe(el);
      });
    }
  }, {
    key: '_observerHandler',
    value: function _observerHandler(entries, observer) {
      var _this2 = this;

      entries.forEach(function (entry) {
        if (entry.isIntersecting) {
          _this2.ListenerQueue.forEach(function (listener) {
            if (listener.el === entry.target) {
              if (listener.loaded) {
                return _this2._observer.unobserve(listener.el);
              }
              listener.loaded = true;

              var image = new Image();
              image.src = listener.binding.value;
              if (listener.binding.arg === 'backgroundImage') {
                image.onload = function () {
                  listener.el.style.backgroundImage = 'url(' + listener.binding.value + ')';
                  listener.el.style.opacity = 1;
                };
              } else {
                image.onload = function () {
                  listener.el.style.backgroundColor = 'transparent';
                  var img = listener.el.querySelector('img');
                  img.src = listener.binding.value;
                  img.style.opacity = 1;
                };
                image.onerror = function () {};
              }
            }
          });
        }
      });
    }
  }]);
  return Lazy;
}();

//
var script = {
  name: 'ImgLazy',
  install: function install(Vue) {

    var lazy = new Lazy(Vue);

    Vue.directive('lazy', {
      bind: lazy.add.bind(lazy)
    });
  },

  props: {
    imgUrl: {
      default: '',
      type: String
    },
    bgColor: {
      default: '#fff',
      type: String
    }
  },
  mounted: function mounted() {}
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
var __vue_script__ = script;

/* template */
var __vue_render__ = function __vue_render__() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c("div", {
    directives: [{
      name: "lazy",
      rawName: "v-lazy",
      value: _vm.imgUrl,
      expression: "imgUrl"
    }],
    staticClass: "taptap-lazy-load-container",
    style: {
      backgroundColor: _vm.bgColor
    }
  }, [_c("img", { staticClass: "taptap-lazy-load-img" })]);
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

/* style */
var __vue_inject_styles__ = undefined;
/* scoped */
var __vue_scope_id__ = undefined;
/* module identifier */
var __vue_module_identifier__ = undefined;
/* functional template */
var __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__ = normalizeComponent({ render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ }, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);

var v_lazy = {
  install: function install(Vue) {

    var lazy = new Lazy(Vue);

    Vue.directive('lazy', {
      bind: lazy.add.bind(lazy)
    });
  }
};

var img_lazy = {
  install: function install() {
    Vue.component(__vue_component__.name, __vue_component__);
  }
};

exports.img_lazy = img_lazy;
exports.v_lazy = v_lazy;
